﻿using System;
using System.Collections.Generic;
using System.Text;

using Com.Piwigo.Lib.DTO;
using Com.Piwigo.Lib.Proxy;
using Com.Piwigo.Lib.Proxy.Response;
using Com.Piwigo.Lib.DTO.Helper;

namespace Com.Piwigo.Lib.Service
{

    static public class PwgTagsService
    {
        /// <summary>
        /// Get the list of tags the user is connected can access
        /// </summary>
        /// <param name="sortedByCounter"></param>
        /// <returns></returns>
        static public List<PwgTag> GetListOfTag(Boolean sortedByCounter)
        {
            List<PwgTag> returnValue = new List<PwgTag>();

            try
            {
                PwgTagsProxyResponse response = PwgTagsProxy.pwg_tags_getList(sortedByCounter);

                if (response.Retour != PwgBaseProxyReponseRetourEnum.Ok)
                {
                    if (response.Erreur != null)
                    {
                        throw new PwgServiceException("GetListOfPwgTag, the server has return the error.",
                            response.Erreur.Code,
                            response.Erreur.Message);
                    }
                    else
                    {
                        throw new PwgServiceException("GetListOfPwgTag : a error occurs during server process.");
                    }
                }
                else
                {
                    returnValue = ConvertProxyResponseToDTO(response);
                }
            }
            catch (PwgProxyException ex)
            {
                throw new PwgServiceException("GetListOfPwgTag : a error is raised by proxy.", ex);
            }
            return returnValue;
        }

        /// <summary>
        /// Get the list of all atgs in piwigo, you must be connected with a high level user
        /// </summary>
        /// <param name="sortedByCounter"></param>
        /// <returns></returns>
        static public List<PwgTag> GetAdminListOfTag()
        {
            List<PwgTag> returnValue = new List<PwgTag>();

            try
            {
                PwgTagsProxyResponse response = PwgTagsProxy.pwg_tags_getAdminList();

                if (response.Retour != PwgBaseProxyReponseRetourEnum.Ok)
                {
                    if (response.Erreur != null)
                    {
                        throw new PwgServiceException("GetAdminListOfTag, the server has return the error.",
                            response.Erreur.Code,
                            response.Erreur.Message);
                    }
                    else
                    {
                        throw new PwgServiceException("GetAdminListOfTag : a error occurs during server process.");
                    }
                }
                else
                {
                    returnValue = ConvertProxyResponseToDTO(response);
                }
            }
            catch (PwgProxyException ex)
            {
                throw new PwgServiceException("GetAdminListOfTag : a error is raised by proxy.", ex);
            }
            return returnValue;
        }


        static public Boolean AddTag(String tagName)
        {
            Boolean returnValue = false;

            try
            {
                PwgTagsProxyResponse response = PwgTagsProxy.pwg_tag_add(tagName);

                if (response.Retour != PwgBaseProxyReponseRetourEnum.Ok)
                {
                    if (response.Erreur != null)
                    {
                        throw new PwgServiceException("AddTag, the server has return the error.",
                            response.Erreur.Code,
                            response.Erreur.Message);
                    }
                    else
                    {
                        throw new PwgServiceException("AddTag : a error occurs during server process.");
                    }
                }
                else
                {
                    returnValue = true;
                }
            }
            catch (PwgProxyException ex)
            {
                throw new PwgServiceException("AddTag : a error is raised by proxy.", ex);
            }
            return returnValue;
        }

        /// <summary>
        /// private: convert response to dto object
        /// </summary>
        /// <param name="response"></param>
        /// <param name="session"></param>
        static private List<PwgTag> ConvertProxyResponseToDTO(PwgTagsProxyResponse response)
        {
            List<PwgTag> returnValue = new List<PwgTag>();

            foreach (PwgTagProxyResponse respTag in response.Tags)
            {
                returnValue.Add(ConvertProxyResponseToDTO(respTag));
            }

            return returnValue;
        }

        /// <summary>
        /// private: convert response to dto object
        /// </summary>
        /// <param name="response"></param>
        /// <param name="session"></param>
        static private PwgTag ConvertProxyResponseToDTO(PwgTagProxyResponse response)
        {
            PwgTag returnValue = new PwgTag();

            try
            {
                returnValue.Counter = response.Counter;
                returnValue.Id = response.Id;
                returnValue.Name = response.Name;
                returnValue.UrlName = response.UrlName;

                if (String.IsNullOrEmpty(response.UrlTag))
                {
                    returnValue.UrlTag = null;
                }
                else
                {
                    returnValue.UrlTag = (new UriBuilder(response.UrlTag)).Uri;
                }
            }
            catch (Exception ex)
            {
                throw new PwgServiceException("ConvertProxyResponseToDTO : a error is raised when converting PwgTagProxyResponse.", ex);
            }

            return returnValue;
        }
    }
}
